@@ -14,7 +14,7 @@ if(Sys.getenv("NO_GPBOOST_ALGO_TESTS") != "NO_GPBOOST_ALGO_TESTS"){
                                          cg_max_num_it = 1000,
                                          cg_max_num_it_tridiag = 1000,
                                          num_rand_vec_trace = 1000,
-                                         reuse_rand_vec_trace = T)
+                                         reuse_rand_vec_trace = TRUE)
   OPTIM_PARAMS_GRAD_DESC <- list(optimizer_cov = "gradient_descent",
                                  lr_cov = 0.1, use_nesterov_acc = TRUE,
                                  acc_rate_cov = 0.5, delta_rel_conv = 1E-6,
@@ -225,7 +225,8 @@ if(Sys.getenv("NO_GPBOOST_ALGO_TESTS") != "NO_GPBOOST_ALGO_TESTS"){
                       use_gp_model_for_validation = TRUE, folds = folds, verbose = 0)
       expect_equal(cvbst$best_iter, 59)
       expect_lt(abs(cvbst$best_score-0.6526893), TOLERANCE)
-      # Parameter tuning
+      
+      # Parameter tuning with 'gpb.grid.search.tune.parameters'
       param_grid = list("learning_rate" = c(1,0.1), 
                         "min_data_in_leaf" = c(10,100))
       other_params <- list(objective = "regression_l2", max_depth = 6, num_leaves = 2^10)
@@ -261,7 +262,43 @@ if(Sys.getenv("NO_GPBOOST_ALGO_TESTS") != "NO_GPBOOST_ALGO_TESTS"){
       expect_equal(opt_params$best_params$min_data_in_leaf, 10)
       expect_equal(opt_params$best_iter, 7)
       expect_lt(abs(opt_params$best_score-1.224379), TOLERANCE)
-      dtrain <- gpb.Dataset(data = X_train, label = y_train)
+      
+      # Parameter tuning with 'tune.pars.bayesian.optimization'
+      suppressWarnings({
+        source("https://raw.githubusercontent.com/fabsig/GPBoost/master/helpers/R_package_tune_pars_bayesian_optimization.R")# Load required function
+      })
+      other_params <- list(objective = "regression_l2", max_depth = 6, num_leaves = 2^10)
+      search_space = list("learning_rate" = c(0.1,1))
+      crit = makeMBOInfillCritCB() # other criterion options: makeMBOInfillCritEI()
+      set.seed(4)
+      opt_params <- tune.pars.bayesian.optimization(search_space = search_space, params = other_params, n_iter = 2,
+                                                    data = dtrain, gp_model = gp_model,
+                                                    nfold = 5, nrounds = 1000, early_stopping_rounds = 10,
+                                                    metric = "l2", crit = crit, cv_seed = 4, verbose_eval = 0)
+      expect_lt(abs(opt_params$best_params$learning_rate-0.1074626), TOLERANCE)
+      expect_equal(opt_params$best_iter, 7)
+      expect_lt(abs(opt_params$best_score-0.09471095), TOLERANCE)
+      # Parameter tuning: can catch errors
+      search_space = list("learning_rate" = c(-1,1))
+      crit = makeMBOInfillCritCB() # other criterion options: makeMBOInfillCritEI()
+      set.seed(4)
+      expect_error({
+        opt_params <- tune.pars.bayesian.optimization(search_space = search_space, params = other_params, n_iter = 2,
+                                                      data = dtrain, gp_model = gp_model,
+                                                      nfold = 5, nrounds = 1000, early_stopping_rounds = 10,
+                                                      metric = "l2", crit = crit, cv_seed = 4, verbose_eval = 0)
+      })
+      # Using 'test_neg_log_likelihood' as metric
+      search_space = list("learning_rate" = c(0.1,1))
+      crit = makeMBOInfillCritCB() # other criterion options: makeMBOInfillCritEI()
+      set.seed(4)
+      opt_params <- tune.pars.bayesian.optimization(search_space = search_space, params = other_params, n_iter = 2,
+                                                    data = dtrain, gp_model = gp_model,
+                                                    nfold = 5, nrounds = 1000, early_stopping_rounds = 10,
+                                                    metric = "test_neg_log_likelihood", crit = crit, cv_seed = 4, verbose_eval = 0)
+      expect_lt(abs(opt_params$best_params$learning_rate-0.1726872), TOLERANCE)
+      expect_equal(opt_params$best_iter, 5)
+      expect_lt(abs(opt_params$best_score-0.3935934), TOLERANCE)
       
       ## Prediction when having only one grouped random effect
       group_1 <- rep(1,ntrain) # grouping variable
